/***************************************************************************
 *   Copyright (C) 2007 by    *
 *      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "acsdoorso.h"
#include <qtango.h>

	char tmBuffer[80];
/*
  timestamp() 
  send a timestamp to standard output
*/
void timestamp() 
{
	struct tm *mytime;
	time_t myTime_t;
	myTime_t = time(NULL);
	mytime = localtime(&myTime_t);
	strftime(tmBuffer, 79, "%Y-%m-%d %H:%M:%S ", mytime);
	return;
}

acsdoorso::acsdoorso(QWidget *parent) : QWidget(parent)
{
	// load (or set) path for images
	QSettings settings("fermi", "acs");
	if (settings.contains("imgpath")) {
		imgpath = settings.value("imgpath").toString();
	}
	else {
		imgpath = "../img";
	}
	if (!imgpath.endsWith("/")) {
		imgpath = imgpath + "/";
	}
	// setup User Interface
	try {
		ui.setupUi(this);
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "cannot setup UI" << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception, cannot setup UI" << endl;
	}
	// connect to control access server
	try {
		plc_server = new Tango::DeviceProxy("f/access_control/safety");
		// plc_server = new Tango::DeviceProxy("ken/plc/pss");
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "Fatal Error: Access Control Device Server not responding, the application will be terminated immediately" << endl;
		exit(1);
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "Fatal Error: Access Control Device Server not responding, the application will be terminated immediately" << endl;
		exit(1);
	}
	refresh();
	// init timer
	try {
		timer = new QTimer(this);
		timer->setInterval(1000);
		timer->setSingleShot(false);
		connect(timer, SIGNAL(timeout()), this, SLOT(refresh()));
		timer->start();
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "cannot connect UI" << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception, cannot setup UI" << endl;
	}
	connect(ui.dr2_Restore, SIGNAL(clicked()), this, SLOT(Restore_i1_so()));
	connect(ui.dr4_Restore, SIGNAL(clicked()), this, SLOT(Restore_i1_e4()));
	connect(ui.dr5_Restore, SIGNAL(clicked()), this, SLOT(Restore_i2_e4()));
	connect(ui.dr7_Restore, SIGNAL(clicked()), this, SLOT(Restore_i1_e5()));
	connect(ui.dr8_Restore, SIGNAL(clicked()), this, SLOT(Restore_i2_e5()));
	connect(ui.dr10_Restore, SIGNAL(clicked()), this, SLOT(Restore_i1_e6()));
	connect(ui.dr11_Restore, SIGNAL(clicked()), this, SLOT(Restore_i2_e6()));
	connect(ui.dr13_Restore, SIGNAL(clicked()), this, SLOT(Restore_i1_e7()));
}

acsdoorso::~acsdoorso()
{
}    
    
int acsdoorso::myReadBoolArray(const char *attrName, vector<bool> & attrVal)
{
	try {
		plc_server->read_attribute(attrName) >> attrVal;
		return 1;
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "error reading attribute: " << attrName << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception reading attribute: " << attrName << endl;
	}
	return 0;
}

int acsdoorso::myReadAttribute(const char *attrName)
{
	try {
		myAttribute = plc_server->read_attribute(attrName);
		return 1;
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << "error reading attribute: " << attrName << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception reading attribute: " << attrName << endl;
	}
	return 0;
}

void acsdoorso::Restore_i1_so()
{
        Tango::DeviceData data_in, data_out;
        Tango::DevShort ch;
	ch = 0;
	// open procedure
	try {
		data_in << ch;
		data_out = plc_server->command_inout("UndulatorAckDoor", data_in);
		return;
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << "error writing command: LinacAckDoor, desc: " << e.errors[0].desc << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception writing command" << endl;
	}
}

void acsdoorso::Restore_i1_e4()
{
        Tango::DeviceData data_in, data_out;
        Tango::DevShort ch;
	ch = 2;
	// open procedure
	try {
		data_in << ch;
		data_out = plc_server->command_inout("UndulatorAckDoor", data_in);
		return;
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << "error writing command: LinacAckDoor, desc: " << e.errors[0].desc << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception writing command" << endl;
	}
}

void acsdoorso::Restore_i2_e4()
{
        Tango::DeviceData data_in, data_out;
        Tango::DevShort ch;
	ch = 1;
	// open procedure
	try {
		data_in << ch;
		data_out = plc_server->command_inout("UndulatorAckDoor", data_in);
		return;
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << "error writing command: LinacAckDoor, desc: " << e.errors[0].desc << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception writing command" << endl;
	}
}

void acsdoorso::Restore_i1_e5()
{
        Tango::DeviceData data_in, data_out;
        Tango::DevShort ch;
	ch = 4;
	// open procedure
	try {
		data_in << ch;
		data_out = plc_server->command_inout("UndulatorAckDoor", data_in);
		return;
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << "error writing command: LinacAckDoor, desc: " << e.errors[0].desc << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception writing command" << endl;
	}
}

void acsdoorso::Restore_i2_e5()
{
        Tango::DeviceData data_in, data_out;
        Tango::DevShort ch;
	ch = 3;
	// open procedure
	try {
		data_in << ch;
		data_out = plc_server->command_inout("UndulatorAckDoor", data_in);
		return;
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << "error writing command: LinacAckDoor, desc: " << e.errors[0].desc << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception writing command" << endl;
	}
}

void acsdoorso::Restore_i1_e6()
{
        Tango::DeviceData data_in, data_out;
        Tango::DevShort ch;
	ch = 5;
	// open procedure
	try {
		data_in << ch;
		data_out = plc_server->command_inout("UndulatorAckDoor", data_in);
		return;
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << "error writing command: LinacAckDoor, desc: " << e.errors[0].desc << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception writing command" << endl;
	}
}

void acsdoorso::Restore_i2_e6()
{
        Tango::DeviceData data_in, data_out;
        Tango::DevShort ch;
	ch = 6;
	// open procedure
	try {
		data_in << ch;
		data_out = plc_server->command_inout("UndulatorAckDoor", data_in);
		return;
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << "error writing command: LinacAckDoor, desc: " << e.errors[0].desc << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception writing command" << endl;
	}
}

void acsdoorso::Restore_i1_e7()
{
        Tango::DeviceData data_in, data_out;
        Tango::DevShort ch;
	ch = 7;
	// open procedure
	try {
		data_in << ch;
		data_out = plc_server->command_inout("UndulatorAckDoor", data_in);
		return;
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << "error writing command: LinacAckDoor, desc: " << e.errors[0].desc << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception writing command" << endl;
	}
}

void acsdoorso::refresh()
{
	vector<bool> stat, stat2;
	// cout << "refresh()" << endl; return;
	// stat.size(16);
	Tango::DeviceAttribute StateFlags;
	// set escape doors
	if (myReadBoolArray("Undulator_doors_so", stat) && myReadBoolArray("Undulator_doors_memoopen_so", stat2)) {
		// main doors
		ui.dr_0->setPixmap(QPixmap(imgpath+(!stat[0]? "door_small_open.png": (stat2[0]? "door_small_hold.png": "door_small_close.png"))));
		ui.dr_1->setPixmap(QPixmap(imgpath+(!stat[1]? "door_small_open.png": (stat2[1]? "door_small_hold.png": "door_small_close.png"))));
		ui.dr_2->setPixmap(QPixmap(imgpath+(!stat[2]? "door_small_open.png": (stat2[2]? "door_small_hold.png": "door_small_close.png"))));
		ui.dr2_Restore->setIcon(QPixmap(imgpath+((stat[2] && stat2[2])? "/ack.png": "/ack_inhibit.png")));
	}
	else {
		// main doors
		ui.dr_0->setPixmap(QPixmap(imgpath+"door_small_unknown.png"));
		ui.dr_1->setPixmap(QPixmap(imgpath+"door_small_unknown.png"));
		ui.dr_2->setPixmap(QPixmap(imgpath+"door_small_unknown.png"));
		ui.dr2_Restore->setIcon(QPixmap(imgpath+"/ack_inhibit.png"));
	}
	// set escape doors
	if (myReadBoolArray("Undulator_doors_e4", stat) && myReadBoolArray("Undulator_doors_memoopen_e4", stat2)) {
		ui.dr_3->setPixmap(QPixmap(imgpath+(!stat[0]? "door_small_open.png": (stat2[0]? "door_small_hold.png": "door_small_close.png"))));
		ui.dr_4->setPixmap(QPixmap(imgpath+(!stat[2]? "door_small_open.png": (stat2[2]? "door_small_hold.png": "door_small_close.png"))));
		ui.dr_5->setPixmap(QPixmap(imgpath+(!stat[3]? "door_small_open.png": (stat2[3]? "door_small_hold.png": "door_small_close.png"))));
		ui.dr4_Restore->setIcon(QPixmap(imgpath+((stat[2] && stat2[2])? "/ack.png": "/ack_inhibit.png")));
		ui.dr5_Restore->setIcon(QPixmap(imgpath+((stat[3] && stat2[3])? "/ack.png": "/ack_inhibit.png")));
	}
	else {
		// main doors
		ui.dr_3->setPixmap(QPixmap(imgpath+"door_small_unknown.png"));
		ui.dr_4->setPixmap(QPixmap(imgpath+"door_small_unknown.png"));
		ui.dr_5->setPixmap(QPixmap(imgpath+"door_small_unknown.png"));
		ui.dr4_Restore->setIcon(QPixmap(imgpath+"/ack_inhibit.png"));
		ui.dr5_Restore->setIcon(QPixmap(imgpath+"/ack_inhibit.png"));
	}
	// set escape doors
	if (myReadBoolArray("Undulator_doors_e5", stat) && myReadBoolArray("Undulator_doors_memoopen_e5", stat2)) {
		ui.dr_6->setPixmap(QPixmap(imgpath+(!stat[0]? "door_small_open.png": (stat2[0]? "door_small_hold.png": "door_small_close.png"))));
		ui.dr_7->setPixmap(QPixmap(imgpath+(!stat[2]? "door_small_open.png": (stat2[2]? "door_small_hold.png": "door_small_close.png"))));
		ui.dr_8->setPixmap(QPixmap(imgpath+(!stat[3]? "door_small_open.png": (stat2[3]? "door_small_hold.png": "door_small_close.png"))));
		ui.dr7_Restore->setIcon(QPixmap(imgpath+((stat[2] && stat2[2])? "/ack.png": "/ack_inhibit.png")));
		ui.dr8_Restore->setIcon(QPixmap(imgpath+((stat[3] && stat2[3])? "/ack.png": "/ack_inhibit.png")));
	}
	else {
		// main doors
		ui.dr_6->setPixmap(QPixmap(imgpath+"door_small_unknown.png"));
		ui.dr_7->setPixmap(QPixmap(imgpath+"door_small_unknown.png"));
		ui.dr_8->setPixmap(QPixmap(imgpath+"door_small_unknown.png"));
		ui.dr7_Restore->setIcon(QPixmap(imgpath+"/ack_inhibit.png"));
		ui.dr8_Restore->setIcon(QPixmap(imgpath+"/ack_inhibit.png"));
	}
	// set escape doors
	if (myReadBoolArray("Undulator_doors_e6", stat) && myReadBoolArray("Undulator_doors_memoopen_e6", stat2)) {
		ui.dr_9->setPixmap(QPixmap(imgpath+(!stat[0]? "door_small_open.png": (stat2[0]? "door_small_hold.png": "door_small_close.png"))));
		ui.dr_10->setPixmap(QPixmap(imgpath+(!stat[2]? "door_small_open.png": (stat2[2]? "door_small_hold.png": "door_small_close.png"))));
		ui.dr_11->setPixmap(QPixmap(imgpath+(!stat[3]? "door_small_open.png": (stat2[3]? "door_small_hold.png": "door_small_close.png"))));
		ui.dr10_Restore->setIcon(QPixmap(imgpath+((stat[2] && stat2[2])? "/ack.png": "/ack_inhibit.png")));
		ui.dr11_Restore->setIcon(QPixmap(imgpath+((stat[3] && stat2[3])? "/ack.png": "/ack_inhibit.png")));
	}
	else {
		// main doors
		ui.dr_9->setPixmap(QPixmap(imgpath+"door_small_unknown.png"));
		ui.dr_10->setPixmap(QPixmap(imgpath+"door_small_unknown.png"));
		ui.dr_11->setPixmap(QPixmap(imgpath+"door_small_unknown.png"));
		ui.dr10_Restore->setIcon(QPixmap(imgpath+"/ack_inhibit.png"));
		ui.dr11_Restore->setIcon(QPixmap(imgpath+"/ack_inhibit.png"));
	}
	// set escape doors
	if (myReadBoolArray("Undulator_doors_e7", stat) && myReadBoolArray("Undulator_doors_memoopen_e7", stat2)) {
		ui.dr_12->setPixmap(QPixmap(imgpath+(!stat[0]? "door_small_open.png": (stat2[0]? "door_small_hold.png": "door_small_close.png"))));
		ui.dr_13->setPixmap(QPixmap(imgpath+(!stat[2]? "door_small_open.png": (stat2[2]? "door_small_hold.png": "door_small_close.png"))));
		ui.dr13_Restore->setIcon(QPixmap(imgpath+((stat[2] && stat2[2])? "/ack.png": "/ack_inhibit.png")));
	}
	else {
		// main doors
		ui.dr_12->setPixmap(QPixmap(imgpath+"door_small_unknown.png"));
		ui.dr_13->setPixmap(QPixmap(imgpath+"door_small_unknown.png"));
		ui.dr13_Restore->setIcon(QPixmap(imgpath+"/ack_inhibit.png"));
	}
}
