/***************************************************************************
 *   Copyright (C) 2007 by    *
 *      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "acsdoorso.h"
#include <qtango.h>

	char tmBuffer[80];
/*
  timestamp() 
  send a timestamp to standard output
*/
void timestamp() 
{
	struct tm *mytime;
	time_t myTime_t;
	myTime_t = time(NULL);
	mytime = localtime(&myTime_t);
	strftime(tmBuffer, 79, "%Y-%m-%d %H:%M:%S ", mytime);
	return;
}

acsdoorso::acsdoorso(QWidget *parent) : QWidget(parent)
{
	// load (or set) path for images
	QSettings settings("elettra", "acs");
	if (settings.contains("imgpath")) {
		imgpath = settings.value("imgpath").toString();
	}
	else {
		imgpath = "../img";
	}
	if (!imgpath.endsWith("/")) {
		imgpath = imgpath + "/";
	}
	// setup User Interface
	try {
		ui.setupUi(this);
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "cannot setup UI" << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception, cannot setup UI" << endl;
	}
	// connect to control access server
	try {
		// plc_server = new Tango::DeviceProxy("ken/plcaccess/0");
		plc_server = new Tango::DeviceProxy("ken/plc/fsafety");
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "Fatal Error: Access Control Device Server not responding, the application will be terminated immediately" << endl;
		exit(1);
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "Fatal Error: Access Control Device Server not responding, the application will be terminated immediately" << endl;
		exit(1);
	}
	refresh();
	// init timer
	try {
		timer = new QTimer(this);
		timer->setInterval(1000);
		timer->setSingleShot(false);
		connect(timer, SIGNAL(timeout()), this, SLOT(refresh()));
		timer->start();
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "cannot connect UI" << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception, cannot setup UI" << endl;
	}
}

acsdoorso::~acsdoorso()
{
}    
    
int acsdoorso::myReadBoolArray(const char *attrName, vector<bool> & attrVal)
{
	try {
		plc_server->read_attribute(attrName) >> attrVal;
		return 1;
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "error reading attribute: " << attrName << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception reading attribute: " << attrName << endl;
	}
	return 0;
}

int acsdoorso::myReadAttribute(const char *attrName)
{
	try {
		myAttribute = plc_server->read_attribute(attrName);
		return 1;
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << "error reading attribute: " << attrName << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception reading attribute: " << attrName << endl;
	}
	return 0;
}

void acsdoorso::refresh()
{
	vector<bool> stat, stat2;
	// cout << "refresh()" << endl; return;
	// stat.size(16);
	Tango::DeviceAttribute StateFlags;
	// set escape doors
	if (myReadBoolArray("Undulator_doors_so", stat) && myReadBoolArray("Undulator_doors_memoopen_so", stat2)) {
		// main doors
		ui.dr_0->setPixmap(QPixmap(imgpath+(!stat[0]? "door_small_open.png": (stat2[0]? "door_small_hold.png": "door_small_close.png"))));
		ui.dr_1->setPixmap(QPixmap(imgpath+(!stat[1]? "door_small_open.png": (stat2[1]? "door_small_hold.png": "door_small_close.png"))));
		ui.dr_2->setPixmap(QPixmap(imgpath+(!stat[2]? "door_small_open.png": (stat2[2]? "door_small_hold.png": "door_small_close.png"))));
		ui.dr_3->setPixmap(QPixmap(imgpath+(!stat[3]? "door_small_open.png": (stat2[3]? "door_small_hold.png": "door_small_close.png"))));
		ui.i1_e1Restore->setIcon(QPixmap(imgpath+((stat[2] && stat2[2])? "/ack.png": "/ack_inhibit.png")));
		ui.i2_e1Restore->setIcon(QPixmap(imgpath+((stat[3] && stat2[3])? "/ack.png": "/ack_inhibit.png")));
	}
	else {
		// main doors
		ui.dr_0->setPixmap(QPixmap(imgpath+"door_small_unknown.png"));
		ui.dr_1->setPixmap(QPixmap(imgpath+"door_small_unknown.png"));
		ui.dr_2->setPixmap(QPixmap(imgpath+"door_small_unknown.png"));
		ui.dr_3->setPixmap(QPixmap(imgpath+"door_small_unknown.png"));
		ui.i1_e1Restore->setIcon(QPixmap(imgpath+"/ack_inhibit.png"));
		ui.i2_e1Restore->setIcon(QPixmap(imgpath+"/ack_inhibit.png"));
	}
}
